@@ -1,5 +1,5 @@
 /* snac - A simple, minimalistic ActivityPub instance */
-/* copyright (c) 2022 - 2024 grunfink et al. / MIT license */
+/* copyright (c) 2022 - 2025 grunfink et al. / MIT license */
 
 #include "xs.h"
 #include "xs_regex.h"
@@ -7,6 +7,7 @@
 #include "xs_html.h"
 #include "xs_json.h"
 #include "xs_time.h"
+#include "xs_match.h"
 
 #include "snac.h"
 
@@ -77,6 +78,8 @@ xs_dict *emojis(void)
     return d;
 }
 
+/* Non-whitespace without trailing comma, period or closing paren */
+#define NOSPACE "([^[:space:],.)]+|[,.)]+[^[:space:],.)])+"
 
 static xs_str *format_line(const char *line, xs_list **attach)
 /* formats a line */
@@ -91,9 +94,12 @@ static xs_str *format_line(const char *line, xs_list **attach)
             "`[^`]+`"                           "|"
             "~~[^~]+~~"                         "|"
             "\\*\\*?\\*?[^\\*]+\\*?\\*?\\*"     "|"
+            "_[^_]+_"                           "|" //anzu
+            "__[^_]+__"                         "|" //anzu
             "!\\[[^]]+\\]\\([^\\)]+\\)"         "|"
             "\\[[^]]+\\]\\([^\\)]+\\)"          "|"
-            "https?:/" "/[^[:space:]]+"
+            "[a-z]+:/" "/" NOSPACE              "|"
+            "(mailto|xmpp):[^@[:space:]]+@" NOSPACE
         ")");
     int n = 0;
 
@@ -125,6 +131,20 @@ static xs_str *format_line(const char *line, xs_list **attach)
                 xs *s2 = xs_fmt("<i>%s</i>", s1);
                 s = xs_str_cat(s, s2);
             }
+            //anzu - begin
+            else
+            if (xs_startswith(v, "__")) {
+                xs *s1 = xs_strip_chars_i(xs_dup(v), "_");
+                xs *s2 = xs_fmt("<u>%s</u>", s1);
+                s = xs_str_cat(s, s2);
+            }
+            else
+            if (xs_startswith(v, "_")) {
+                xs *s1 = xs_strip_chars_i(xs_dup(v), "_");
+                xs *s2 = xs_fmt("<i>%s</i>", s1);
+                s = xs_str_cat(s, s2);
+            }
+            //anzu - end
             else
             if (xs_startswith(v, "~~")) {
                 xs *s1 = xs_strip_chars_i(xs_dup(v), "~");
@@ -133,32 +153,11 @@ static xs_str *format_line(const char *line, xs_list **attach)
                 s = xs_str_cat(s, s2);
             }
             else
-            if (xs_startswith(v, "http")) {
-                xs *u  = xs_replace(v, "#", "&#35;");
-                xs *v2 = xs_strip_chars_i(xs_dup(u), ".,)");
-
-                const char *mime = xs_mime_by_ext(v2);
-
-                if (attach != NULL && xs_startswith(mime, "image/")) {
-                    /* if it's a link to an image, insert it as an attachment */
-                    xs *d = xs_dict_new();
-
-                    d = xs_dict_append(d, "mediaType", mime);
-                    d = xs_dict_append(d, "url",       v2);
-                    d = xs_dict_append(d, "name",      "");
-                    d = xs_dict_append(d, "type",      "Image");
-
-                    *attach = xs_list_append(*attach, d);
-                }
-                else {
-                    xs *s1 = xs_fmt("<a href=\"%s\" target=\"_blank\">%s</a>", v2, u);
-                    s = xs_str_cat(s, s1);
-                }
-            }
-            else
             if (*v == '[') {
                 /* markdown-like links [label](url) */
-                xs *w = xs_strip_chars_i(xs_replace(v, "#", "&#35;"), "[)");
+                xs *w = xs_strip_chars_i(
+                    xs_replace_i(xs_replace(v, "#", "&#35;"), "@", "&#64;"),
+                "![)");
                 xs *l = xs_split_n(w, "](", 1);
 
                 if (xs_list_len(l) == 2) {
@@ -173,7 +172,9 @@ static xs_str *format_line(const char *line, xs_list **attach)
             else
             if (*v == '!') {
                 /* markdown-like images ![alt text](url to image) */
-                xs *w = xs_strip_chars_i(xs_replace(v, "#", "&#35;"), "![)");
+                xs *w = xs_strip_chars_i(
+                    xs_replace_i(xs_replace(v, "#", "&#35;"), "@", "&#64;"),
+                "![)");
                 xs *l = xs_split_n(w, "](", 1);
 
                 if (xs_list_len(l) == 2) {
@@ -182,14 +183,26 @@ static xs_str *format_line(const char *line, xs_list **attach)
                     const char *mime     = xs_mime_by_ext(img_url);
 
                     if (attach != NULL && xs_startswith(mime, "image/")) {
-                        xs *d = xs_dict_new();
+                        const xs_dict *ad;
+                        int add = 1;
 
-                        d = xs_dict_append(d, "mediaType", mime);
-                        d = xs_dict_append(d, "url",       img_url);
-                        d = xs_dict_append(d, "name",      alt_text);
-                        d = xs_dict_append(d, "type",      "Image");
+                        xs_list_foreach(*attach, ad) {
+                            if (strcmp(xs_dict_get_def(ad, "url", ""), img_url) == 0) {
+                                add = 0;
+                                break;
+                            }
+                        }
 
-                        *attach = xs_list_append(*attach, d);
+                        if (add) {
+                            xs *d = xs_dict_new();
+
+                            d = xs_dict_append(d, "mediaType", mime);
+                            d = xs_dict_append(d, "url",       img_url);
+                            d = xs_dict_append(d, "name",      alt_text);
+                            d = xs_dict_append(d, "type",      "Image");
+
+                            *attach = xs_list_append(*attach, d);
+                        }
                     }
                     else {
                         xs *link = xs_fmt("<a href=\"%s\">%s</a>", img_url, alt_text);
@@ -200,6 +213,51 @@ static xs_str *format_line(const char *line, xs_list **attach)
                 else
                     s = xs_str_cat(s, v);
             }
+            else
+            if (xs_str_in(v, ":/" "/") != -1) {
+                xs *u  = xs_replace_i(xs_replace(v, "#", "&#35;"), "@", "&#64;");
+
+                xs *v2 = xs_strip_chars_i(xs_dup(u), ".,)");
+
+                const char *mime = xs_mime_by_ext(v2);
+
+                if (attach != NULL && xs_startswith(mime, "image/")) {
+                    /* if it's a link to an image, insert it as an attachment */
+                    const xs_dict *ad;
+                    int add = 1;
+
+                    xs_list_foreach(*attach, ad) {
+                        if (strcmp(xs_dict_get_def(ad, "url", ""), v2) == 0) {
+                            add = 0;
+                            break;
+                        }
+                    }
+
+                    if (add) {
+                        xs *d = xs_dict_new();
+
+                        d = xs_dict_append(d, "mediaType", mime);
+                        d = xs_dict_append(d, "url",       v2);
+                        d = xs_dict_append(d, "name",      "");
+                        d = xs_dict_append(d, "type",      "Image");
+
+                        *attach = xs_list_append(*attach, d);
+                    }
+                }
+                else {
+                    xs *s1 = xs_fmt("<a href=\"%s\" target=\"_blank\">%s</a>", v2, u);
+                    s = xs_str_cat(s, s1);
+                }
+            }
+            else
+            if (xs_match(v, "mailto*|xmpp*")) {
+                xs *u  = xs_replace_i(xs_replace(v, "#", "&#35;"), "@", "&#64;");
+
+                xs *v2 = xs_strip_chars_i(xs_dup(u), ".,)");
+
+                xs *s1 = xs_fmt("<a href=\"%s\" target=\"_blank\">%s</a>", v2, u);
+                s = xs_str_cat(s, s1);
+            }
             else
                 s = xs_str_cat(s, v);
         }
@@ -263,6 +321,31 @@ xs_str *not_really_markdown(const char *content, xs_list **attach, xs_list **tag
             continue;
         }
 
+        //anzu - begin
+        // h1 reserved for snac?
+        if (xs_startswith(ss, "# ")) {
+            ss = xs_strip_i(xs_crop_i(ss, 2, 0));
+            s = xs_str_cat(s, "<h2>");
+            s = xs_str_cat(s, ss);
+            s = xs_str_cat(s, "</h2>");
+            continue;
+        }
+        if (xs_startswith(ss, "## ")) {
+            ss = xs_strip_i(xs_crop_i(ss, 3, 0));
+            s = xs_str_cat(s, "<h2>");
+            s = xs_str_cat(s, ss);
+            s = xs_str_cat(s, "</h2>");
+            continue;
+        }
+        if (xs_startswith(ss, "### ")) {
+            ss = xs_strip_i(xs_crop_i(ss, 4, 0));
+            s = xs_str_cat(s, "<h3>");
+            s = xs_str_cat(s, ss);
+            s = xs_str_cat(s, "</h3>");
+            continue;
+        }
+        //anzu - end
+
         if (xs_startswith(ss, ">")) {
             /* delete the > and subsequent spaces */
             ss = xs_strip_i(xs_crop_i(ss, 1, 0));
@@ -296,6 +379,8 @@ xs_str *not_really_markdown(const char *content, xs_list **attach, xs_list **tag
     s = xs_replace_i(s, "<br><br><blockquote>", "<br><blockquote>");
     s = xs_replace_i(s, "</blockquote><br>", "</blockquote>");
     s = xs_replace_i(s, "</pre><br>", "</pre>");
+    s = xs_replace_i(s, "</h2><br>", "</h2>"); //anzu ???
+    s = xs_replace_i(s, "</h3><br>", "</h3>"); //anzu ???
 
     {
         /* traditional emoticons */
@@ -338,7 +423,9 @@ xs_str *not_really_markdown(const char *content, xs_list **attach, xs_list **tag
 
 const char *valid_tags[] = {
     "a", "p", "br", "br/", "blockquote", "ul", "ol", "li", "cite", "small",
-    "span", "i", "b", "u", "s", "pre", "code", "em", "strong", "hr", "img", "del", "bdi", NULL
+    "span", "i", "b", "u", "s", "pre", "code", "em", "strong", "hr", "img", "del", "bdi",
+    "h2","h3", //anzu
+    NULL
 };
 
 xs_str *sanitize(const char *content)
