@@ -1,5 +1,5 @@
 /* snac - A simple, minimalistic ActivityPub instance */
-/* copyright (c) 2022 - 2024 grunfink et al. / MIT license */
+/* copyright (c) 2022 - 2025 grunfink et al. / MIT license */
 
 #include "xs.h"
 #include "xs_io.h"
@@ -138,7 +138,7 @@ static xs_str *greeting_html(void)
             while (xs_list_iter(&p, &uid)) {
                 snac user;
 
-                if (user_open(&user, uid)) {
+                if (strcmp(uid, "relay") && user_open(&user, uid)) {
                     xs_html_add(ul,
                         xs_html_tag("li",
                             xs_html_tag("a",
@@ -164,6 +164,47 @@ static xs_str *greeting_html(void)
 }
 
 
+const char *share_page = ""
+"<!DOCTYPE html>\n"
+"<html>\n"
+"<head>\n"
+"<title>%s - snac</title>\n"
+"<meta content=\"width=device-width, initial-scale=1, minimum-scale=1, user-scalable=no\" name=\"viewport\">\n"
+"<link rel=\"stylesheet\" type=\"text/css\" href=\"%s/style.css\"/>\n"
+"<style>:root {color-scheme: light dark}</style>\n"
+"</head>\n"
+"<body><h1>%s link share</h1>\n"
+"<form method=\"get\" action=\"%s/share-bridge\">\n"
+"<textarea name=\"content\" rows=\"6\" wrap=\"virtual\" required=\"required\" style=\"width: 50em\">%s</textarea>\n"
+"<p>Login: <input type=\"text\" name=\"login\" autocapitalize=\"off\" required=\"required\"></p>\n"
+"<input type=\"submit\" value=\"OK\">\n"
+"</form><p>%s</p></body></html>\n"
+"";
+
+
+const char *authorize_interaction_page = ""
+"<!DOCTYPE html>\n"
+"<html>\n"
+"<head>\n"
+"<title>%s - snac</title>\n"
+"<meta content=\"width=device-width, initial-scale=1, minimum-scale=1, user-scalable=no\" name=\"viewport\">\n"
+"<link rel=\"stylesheet\" type=\"text/css\" href=\"%s/style.css\"/>\n"
+"<style>:root {color-scheme: light dark}</style>\n"
+"</head>\n"
+"<body><h1>%s authorize interaction</h1>\n"
+"<form method=\"get\" action=\"%s/auth-int-bridge\">\n"
+"<select name=\"action\">\n"
+"<option value=\"Follow\">Follow</option>\n"
+"<option value=\"Boost\">Boost</option>\n"
+"<option value=\"Like\">Like</option>\n"
+"</select> %s\n"
+"<input type=\"hidden\" name=\"id\" value=\"%s\">\n"
+"<p>Login: <input type=\"text\" name=\"login\" autocapitalize=\"off\" required=\"required\"></p>\n"
+"<input type=\"submit\" value=\"OK\">\n"
+"</form><p>%s</p></body></html>\n"
+"";
+
+
 int server_get_handler(xs_dict *req, const char *q_path,
                        char **body, int *b_size, char **ctype)
 /* basic server services */
@@ -171,14 +212,15 @@ int server_get_handler(xs_dict *req, const char *q_path,
     int status = 0;
 
     /* is it the server root? */
-    if (*q_path == '\0') {
+    if (*q_path == '\0' || strcmp(q_path, "/") == 0) {
         const xs_dict *q_vars = xs_dict_get(req, "q_vars");
         const char *t = NULL;
 
         if (xs_type(q_vars) == XSTYPE_DICT && (t = xs_dict_get(q_vars, "t"))) {
             /** search by tag **/
             int skip = 0;
-            int show = xs_number_get(xs_dict_get(srv_config, "max_timeline_entries"));
+            int show = xs_number_get(xs_dict_get_def(srv_config, "def_timeline_entries",
+                                 xs_dict_get_def(srv_config, "max_timeline_entries", "50")));
             const char *v;
 
             if ((v = xs_dict_get(q_vars, "skip")) != NULL)
@@ -204,7 +246,7 @@ int server_get_handler(xs_dict *req, const char *q_path,
             else {
                 xs *page = xs_fmt("?t=%s", t);
                 xs *title = xs_fmt(L("Search results for tag #%s"), t);
-                *body = html_timeline(NULL, tl, 0, skip, show, more, title, page, 0);
+                *body = html_timeline(NULL, tl, 0, skip, show, more, title, page, 0, NULL);
             }
         }
         else
@@ -212,7 +254,7 @@ int server_get_handler(xs_dict *req, const char *q_path,
             /** instance timeline **/
             xs *tl = timeline_instance_list(0, 30);
             *body = html_timeline(NULL, tl, 0, 0, 0, 0,
-                L("Recent posts by users in this instance"), NULL, 0);
+                L("Recent posts by users in this instance"), NULL, 0, NULL);
         }
         else
             *body = greeting_html();
@@ -257,6 +299,68 @@ int server_get_handler(xs_dict *req, const char *q_path,
         *body  = xs_str_new("User-agent: *\n"
                             "Disallow: /\n");
     }
+    else
+    if (strcmp(q_path, "/style.css") == 0) {
+        FILE *f;
+        xs *css_fn = xs_fmt("%s/style.css", srv_basedir);
+
+        if ((f = fopen(css_fn, "r")) != NULL) {
+            *body = xs_readall(f);
+            fclose(f);
+
+            status = HTTP_STATUS_OK;
+            *ctype = "text/css";
+        }
+    }
+    else
+    if (strcmp(q_path, "/share") == 0) {
+        const xs_dict *q_vars = xs_dict_get(req, "q_vars");
+        const char *url  = xs_dict_get(q_vars, "url");
+        const char *text = xs_dict_get(q_vars, "text");
+        xs *s = NULL;
+
+        if (xs_type(text) == XSTYPE_STRING) {
+            if (xs_type(url) == XSTYPE_STRING)
+                s = xs_fmt("%s:\n\n%s\n", text, url);
+            else
+                s = xs_fmt("%s\n", text);
+        }
+        else
+        if (xs_type(url) == XSTYPE_STRING)
+            s = xs_fmt("%s\n", url);
+        else
+            s = xs_str_new(NULL);
+
+        status = HTTP_STATUS_OK;
+        *ctype = "text/html; charset=utf-8";
+        *body  = xs_fmt(share_page,
+            xs_dict_get(srv_config, "host"),
+            srv_baseurl,
+            xs_dict_get(srv_config, "host"),
+            srv_baseurl,
+            s,
+            USER_AGENT
+        );
+    }
+    else
+    if (strcmp(q_path, "/authorize_interaction") == 0) {
+        const xs_dict *q_vars = xs_dict_get(req, "q_vars");
+        const char *uri  = xs_dict_get(q_vars, "uri");
+
+        if (xs_is_string(uri)) {
+            status = HTTP_STATUS_OK;
+            *ctype = "text/html; charset=utf-8";
+            *body  = xs_fmt(authorize_interaction_page,
+                xs_dict_get(srv_config, "host"),
+                srv_baseurl,
+                xs_dict_get(srv_config, "host"),
+                srv_baseurl,
+                uri,
+                uri,
+                USER_AGENT
+            );
+        }
+    }
 
     if (status != 0)
         srv_debug(1, xs_fmt("server_get_handler serving '%s' %d", q_path, status));
@@ -278,6 +382,8 @@ void httpd_connection(FILE *f)
     xs *q_path   = NULL;
     xs *payload  = NULL;
     xs *etag     = NULL;
+    xs *last_modified = NULL;
+    xs *link     = NULL;
     int p_size   = 0;
     const char *p;
     int fcgi_id;
@@ -325,11 +431,11 @@ void httpd_connection(FILE *f)
             status = oauth_get_handler(req, q_path, &body, &b_size, &ctype);
 
         if (status == 0)
-            status = mastoapi_get_handler(req, q_path, &body, &b_size, &ctype);
+            status = mastoapi_get_handler(req, q_path, &body, &b_size, &ctype, &link);
 #endif /* NO_MASTODON_API */
 
         if (status == 0)
-            status = html_get_handler(req, q_path, &body, &b_size, &ctype, &etag);
+            status = html_get_handler(req, q_path, &body, &b_size, &ctype, &etag, &last_modified);
     }
     else
     if (strcmp(method, "POST") == 0) {
@@ -396,13 +502,13 @@ void httpd_connection(FILE *f)
     }
 
     if (status == HTTP_STATUS_FORBIDDEN)
-        body = xs_str_new("<h1>403 Forbidden</h1>");
+        body = xs_str_new("<h1>403 Forbidden (" USER_AGENT ")</h1>");
 
     if (status == HTTP_STATUS_NOT_FOUND)
-        body = xs_str_new("<h1>404 Not Found</h1>");
+        body = xs_str_new("<h1>404 Not Found (" USER_AGENT ")</h1>");
 
     if (status == HTTP_STATUS_BAD_REQUEST && body != NULL)
-        body = xs_str_new("<h1>400 Bad Request</h1>");
+        body = xs_str_new("<h1>400 Bad Request (" USER_AGENT ")</h1>");
 
     if (status == HTTP_STATUS_SEE_OTHER)
         headers = xs_dict_append(headers, "location", body);
@@ -423,6 +529,10 @@ void httpd_connection(FILE *f)
 
     if (!xs_is_null(etag))
         headers = xs_dict_append(headers, "etag", etag);
+    if (!xs_is_null(last_modified))
+        headers = xs_dict_append(headers, "last-modified", last_modified);
+    if (!xs_is_null(link))
+        headers = xs_dict_append(headers, "Link", link);
 
     /* if there are any additional headers, add them */
     const xs_dict *more_headers = xs_dict_get(srv_config, "http_headers");
@@ -771,6 +881,27 @@ void httpd(void)
     xs *sem_name = NULL;
     xs *shm_name = NULL;
     sem_t anon_job_sem;
+    xs *pidfile = xs_fmt("%s/server.pid", srv_basedir);
+    int pidfd;
+
+    {
+        /* do some pidfile locking acrobatics */
+        if ((pidfd = open(pidfile, O_RDWR | O_CREAT, 0660)) == -1) {
+            srv_log(xs_fmt("Cannot create pidfile %s -- cannot continue", pidfile));
+            return;
+        }
+
+        if (lockf(pidfd, F_TLOCK, 1) == -1) {
+            srv_log(xs_fmt("Cannot lock pidfile %s -- server already running?", pidfile));
+            close(pidfd);
+            return;
+        }
+
+        ftruncate(pidfd, 0);
+
+        xs *s = xs_fmt("%d\n", (int)getpid());
+        write(pidfd, s, strlen(s));
+    }
 
     address = xs_dict_get(srv_config, "address");
 
@@ -891,4 +1022,6 @@ void httpd(void)
     srv_log(xs_fmt("httpd%s stop %s (run time: %s)",
                 p_state->use_fcgi ? " (FastCGI)" : "",
                 full_address, uptime));
+
+    unlink(pidfile);
 }
