@@ -28,6 +28,7 @@ static const char *default_srv_config = "{"
     "\"queue_timeout\":        6,"
     "\"queue_timeout_2\":      8,"
     "\"cssurls\":              [\"\"],"
+    "\"def_timeline_entries\": 50,"
     "\"max_timeline_entries\": 50,"
     "\"timeline_purge_days\":  120,"
     "\"local_purge_days\":     0,"
@@ -36,6 +37,7 @@ static const char *default_srv_config = "{"
     "\"admin_account\":        \"\","
     "\"title\":                \"\","
     "\"short_description\":    \"\","
+    "\"short_description_raw\": false,"
     "\"protocol\":             \"https\","
     "\"fastcgi\":              false"
     "}";
