@@ -1,4 +1,4 @@
-/* copyright (c) 2022 - 2024 grunfink et al. / MIT license */
+/* copyright (c) 2022 - 2025 grunfink et al. / MIT license */
 
 #ifndef _XS_JSON_H
 
@@ -75,7 +75,6 @@ static void _xs_json_dump(const xs_val *data, int level, int indent, FILE *f)
 /* dumps partial data as JSON */
 {
     int c = 0;
-    int ct = 0;
     const xs_val *v;
 
     switch (xs_type(data)) {
@@ -98,7 +97,7 @@ static void _xs_json_dump(const xs_val *data, int level, int indent, FILE *f)
     case XSTYPE_LIST:
         fputc('[', f);
 
-        while (xs_list_next(data, &v, &ct)) {
+        xs_list_foreach(data, v) {
             if (c != 0)
                 fputc(',', f);
 
@@ -118,7 +117,7 @@ static void _xs_json_dump(const xs_val *data, int level, int indent, FILE *f)
 
         const xs_str *k;
 
-        while (xs_dict_next(data, &k, &v, &ct)) {
+        xs_dict_foreach(data, k, v) {
             if (c != 0)
                 fputc(',', f);
 
@@ -208,11 +207,12 @@ static xs_val *_xs_json_load_lexer(FILE *f, js_type *t)
 {
     int c;
     xs_val *v = NULL;
+    int offset;
 
     *t = JS_ERROR;
 
     /* skip blanks */
-    while ((c = fgetc(f)) == L' ' || c == L'\t' || c == L'\n' || c == L'\r');
+    while ((c = fgetc(f)) == ' ' || c == '\t' || c == '\n' || c == '\r');
 
     if (c == '{')
         *t = JS_OCURLY;
@@ -236,6 +236,7 @@ static xs_val *_xs_json_load_lexer(FILE *f, js_type *t)
         *t = JS_STRING;
 
         v = xs_str_new(NULL);
+        offset = 0;
 
         while ((c = fgetc(f)) != '"' && c != EOF && *t != JS_ERROR) {
             if (c == '\\') {
@@ -274,11 +275,18 @@ static xs_val *_xs_json_load_lexer(FILE *f, js_type *t)
                     break;
                 }
 
-                v = xs_utf8_cat(v, cp);
+                v = xs_utf8_insert(v, cp, &offset);
             }
             else {
                 char cc = c;
-                v = xs_append_m(v, &cc, 1);
+                v = xs_insert_m(v, offset, &cc, 1);
+
+                if (!xs_is_string(v)) {
+                    *t = JS_ERROR;
+                    break;
+                }
+
+                offset++;
             }
         }
 
